#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2021-12-18
#
# bakery plugin for check_mk
#
# 2021-12-23: reworked options handling
# 2021-12-25: backported for cmk 1.6
#
import os


def get_cve_2021_44228_log4j_files(opsys, conf, conf_dir, plugins_dir):
    options_array = []
    search_path_array = []
    separator = ' '  # needs matching separator in the shell scripts

    options = conf[1].copy()

    interval = options.get('interval', 86400)
    timeout = options.get('timeout', 300)
    site_name = options['site_name']

    search_path = ''
    if conf[0] == 'linux':
        for path in options['search_linux']:
            path = path.strip(' ').strip("'").strip('"')
            search_path_array.append('"%s"' % path)
        search_path = ' '.join(search_path_array)
    elif conf[0] == 'windows':
        if options['search_windows'] == 'all_drives':
            options_array.append('--all-drives')
        elif 'drives_to_scan' in options['search_windows']:
            label, drives_to_scan = options['search_windows']
            drives_to_scan = '--drives %s' % ",".join(drives_to_scan)
            options_array.append(drives_to_scan)
        elif 'search_paths' in options['search_windows']:
            label, search_path = options['search_windows']
            search_path = " ".join(search_path)

    for path in options.get('exclude_paths', []):
        path = path.strip(' ').strip("'").strip('"')
        options_array.append('--exclude "%s"' % path)

    if options.get('exclude_fs'):
        exclude_fs = ','.join(options['exclude_fs'])
        options_array.append('--exclude-fs %s' % exclude_fs)

    if options.get('syslog'):
        options_array.append('--syslog-level %s' % options["syslog"].get("syslog_level", "info"))
        syslog_server = '--syslog-udp %s' % options["syslog"]["syslog_server"]
        syslog_port = options['syslog'].get('syslog_port', None)
        if syslog_port is not None:
            syslog_port = ':%s' % syslog_port
        else:
            syslog_port = ''
        options_array.append('%s%s' % (syslog_server, syslog_port))

    if options.get('reporting'):
        report_dir = options["reporting"]["report_dir"].strip(' ').strip("'").strip('"')
        options_array.append('--report-dir "%s"' % report_dir)
        options_array.append(options['reporting'].get('report_format', '--report-csv'))
        if options['reporting'].get('no_empty_report'):
            options_array.append('--no-empty-report')

    if options.get('fix_files'):
        backup_dir = options["fix_files"]["backup_dir"].strip(' ').strip("'").strip('"')
        options_array.append('--backup-path "%s"' % backup_dir)
        if options['fix_files'].get('not_exclude_backup') is None:
            options_array.append('--exclude "%s"' % backup_dir)
        if options['fix_files'].get('force_fix'):
            options_array.append('--force-fix')

    for key in [
        'interval',
        'timeout',
        'search_linux',
        'search_windows',
        'exclude_paths',
        'exclude_fs',
        'syslog',
        'reporting',
        'fix_files',
        'site_name',
    ]:
        try:
            options.pop(key)
        except KeyError:
            pass

    for value in options.values():
        options_array.append(value)

    options = separator.join(options_array)
    local_plugin_dir = '/omd/sites/%s/local/share/check_mk/agents/plugins' % site_name

    if conf[0] == 'linux' and opsys == 'linux':
        # shutil.copy2(cmk.utils.paths.agents_dir + "/plugins/apache_status",  plugins_dir + "/apache_status")
        target_dir = plugins_dir + "/%s" % interval  # Interval for async execution
        if not os.path.exists(target_dir):
            os.makedirs(target_dir)

        shutil.copy2(local_plugin_dir + "/cve_2021_44228_log4j.linux", target_dir + "/cve_2021_44228_log4j.sh")
        os.chmod(target_dir + "/cve_2021_44228_log4j.sh", 0755)
        shutil.copy2(local_plugin_dir + "/log4j2-scan.linux", plugins_dir + "/../bin/log4j2-scan")
        os.chmod(plugins_dir + "/../bin//log4j2-scan", 0755)


        # yield Plugin(
        #     base_os=OS.LINUX,
        #     source=Path('cve_2021_44228_log4j.linux'),
        #     target=Path('cve_2021_44228_log4j.sh'),
        #     asynchronous=True,
        #     interval=interval,
        #     timeout=timeout,
        # )
        # yield Plugin(
        #     base_os=OS.LINUX,
        #     source=Path('log4j2-scan.linux'),
        #     target=Path('../bin/log4j2-scan'),
        # )

        f = file(conf_dir + "/cve_2021_44228_log4j.cfg", "w")
        f.write(agent_file_header)
        f.write('OPTIONS=(%s%s%s);' % (options, separator, search_path))

        # yield PluginConfig(
        #     base_os=OS.LINUX,
        #     lines=['OPTIONS=(%s%s%s);' % (options, separator, search_path)],
        #     target=Path('cve_2021_44228_log4j.cfg'),
        #     include_header=True,
        # )

    elif conf[0] == 'windows' and opsys == 'windows':
        shutil.copy2(local_plugin_dir + "/cve_2021_44228_log4j.windows", plugins_dir + "/cve_2021_44228_log4j.ps1")
        shutil.copy2(local_plugin_dir + "/log4j2-scan.windows", plugins_dir + "/../bin/log4j2-scan.exe")

        # yield Plugin(
        #     base_os=OS.WINDOWS,
        #     source=Path('cve_2021_44228_log4j.windows'),
        #     target=Path('cve_2021_44228_log4j.ps1'),
        #     asynchronous=True,
        #     interval=interval,
        #     timeout=timeout,
        # )
        # yield Plugin(
        #     base_os=OS.WINDOWS,
        #     source=Path('log4j2-scan.windows'),
        #     target=Path('..\\bin\\log4j2-scan.exe'),
        # )

        f = file(conf_dir + "/cve_2021_44228_log4j.cfg", "w")
        f.write(agent_file_header)
        f.write('OPTIONS=%s%s%s' % (options, separator, search_path))

        # yield PluginConfig(
        #         base_os=OS.WINDOWS,
        #         lines=['OPTIONS=(%s%s%s);' % (options, separator, search_path)],
        #         target=Path('cve_2021_44228_log4j.cfg'),
        #         include_header=True,
        #     )


bakery_info["cve_2021_44228_log4j"] = {
    "bake_function": get_cve_2021_44228_log4j_files,
    "os": ["linux", 'windows'],
    # "verify_python": True,  # preinstall scripts ensure python is available and not python 3.x
}
